@@ -44,11 +44,11 @@ G_BEGIN_DECLS
 #define LIBBRASERO_MEDIA_VERSION_MINOR						\
 	31
 #define LIBBRASERO_MEDIA_VERSION_MICRO						\
-	4
+	90
 #define LIBBRASERO_MEDIA_AGE							\
 	302
 #define LIBBRASERO_MEDIA_INSTALL							\
-	"/usr/local"
+	"/scratch/jhbuild"
 
 /**
  * To start and stop the library
