@@ -42,6 +42,8 @@
 
 #include <gio/gio.h>
 
+#include <gdk/gdkx.h>
+
 #include <gtk/gtk.h>
 
 #ifdef BUILD_PLAYLIST
@@ -100,6 +102,9 @@ struct _BraseroIOPrivate
 
 	guint progress_id;
 	GSList *progress;
+
+	BraseroIOGetParentWinCb win_callback;
+	gpointer win_user_data;
 };
 
 #define BRASERO_IO_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_IO, BraseroIOPrivate))
@@ -182,9 +187,9 @@ brasero_io_job_progress_report_cb (gpointer callback_data)
 
 		/* update our progress */
 		progress->progress (progress->job, progress);
-		progress->job->base->progress (progress->job->base->object,
-					       progress,
-					       callback_data);
+		progress->job->base->methods->progress (progress->job->base->object,
+		                                        progress,
+		                                        callback_data);
 	}
 	g_mutex_unlock (priv->lock);
 
@@ -201,7 +206,7 @@ brasero_io_job_progress_report_start (BraseroIO *self,
 
 	priv = BRASERO_IO_PRIVATE (self);
 
-	if (!job->base->progress)
+	if (!job->base->methods->progress)
 		return;
 
 	progress = g_new0 (BraseroIOJobProgress, 1);
@@ -248,12 +253,6 @@ brasero_io_job_progress_report_stop (BraseroIO *self,
 	g_mutex_unlock (priv->lock);
 }
 
-const gchar *
-brasero_io_job_progress_get_current (BraseroIOJobProgress *progress)
-{
-	return g_strdup (progress->current);
-}
-
 guint
 brasero_io_job_progress_get_file_processed (BraseroIOJobProgress *progress)
 {
@@ -327,8 +326,6 @@ brasero_io_return_result_idle (gpointer callback_data)
 	BraseroIOJobResult *result;
 	BraseroIOPrivate *priv;
 	guint results_id;
-	GSList *iter;
-	GSList *next;
 	int i;
 
 	priv = BRASERO_IO_PRIVATE (self);
@@ -337,62 +334,72 @@ brasero_io_return_result_idle (gpointer callback_data)
 
 	/* Put that to 0 for now so that a new idle call will be scheduled while
 	 * we are in the loop. That way if we block the other one will be able 
-	 * to deliver the results. */
+	 * to deliver results. */
 	results_id = priv->results_id;
 	priv->results_id = 0;
 
 	/* Return several results at a time that can be a huge speed gain.
 	 * What should be the value that provides speed and responsiveness? */
-	for (i = 0, iter = priv->results; iter && i < NUMBER_OF_RESULTS; iter = next) {
+	for (i = 0; priv->results && i < NUMBER_OF_RESULTS;) {
 		BraseroIOJobBase *base;
+		GSList *iter;
 
-		result = iter->data;
-		next = iter->next;
+		/* Find the next result that can be returned */
+		result = NULL;
+		for (iter = priv->results; iter; iter = iter->next) {
+			BraseroIOJobResult *tmp_result;
+
+			tmp_result = iter->data;
+			if (!tmp_result->base->methods->in_use) {
+				result = tmp_result;
+				break;
+			}
+		}
+
+		if (!result)
+			break;
 
 		/* Make sure another result is not returned for this base. This 
 		 * is to avoid BraseroDataDisc showing multiple dialogs for 
 		 * various problems; like one dialog for joliet, one for deep,
 		 * and one for name collision. */
-		if (result->base->in_use)
-			continue;
+		base = (BraseroIOJobBase *) result->base;
+		base->methods->in_use = TRUE;
 
 		priv->results = g_slist_remove (priv->results, result);
 
-		base = (BraseroIOJobBase *) result->base;
-		base->in_use = TRUE;
+		/* This is to make sure the object
+		 *  lives as long as we need it. */
+		g_object_ref (base->object);
 
 		g_mutex_unlock (priv->lock);
 
 		data = result->callback_data;
 
-		/* This is to make sure the object lives
-		 * as long as we need it. */
-		g_object_ref (base->object);
-
 		if (result->uri || result->info || result->error)
-			result->base->callback (base->object,
-						result->error,
-						result->uri,
-						result->info,
-						data? data->callback_data:NULL);
+			base->methods->callback (base->object,
+			                          result->error,
+			                          result->uri,
+			                          result->info,
+			                          data? data->callback_data:NULL);
 
 		/* call destroy () for callback data */
 		brasero_io_unref_result_callback_data (data,
 						       base->object,
-						       base->destroy,
+						       base->methods->destroy,
 						       FALSE);
 
-		g_object_unref (base->object);
-
 		brasero_io_job_result_free (result);
 
 		g_mutex_lock (priv->lock);
 
 		i ++;
-		base->in_use = FALSE;
+
+		g_object_unref (base->object);
+		base->methods->in_use = FALSE;
 	}
 
-	if (!priv->results_id && iter && i >= NUMBER_OF_RESULTS) {
+	if (!priv->results_id && priv->results && i >= NUMBER_OF_RESULTS) {
 		/* There are still results and no idle call is scheduled so we
 		 * have to restart ourselves to make sure we empty the queue */
 		priv->results_id = results_id;
@@ -520,10 +527,10 @@ brasero_io_job_free (gboolean cancelled,
 		 * add a dummy result to destroy callback_data. */
 		if (g_atomic_int_dec_and_test (&job->callback_data->ref)) {
 			if (cancelled) {
-				if (job->base->destroy)
-					job->base->destroy (job->base->object,
-							    TRUE,
-							    job->callback_data->callback_data);
+				if (job->base->methods->destroy)
+					job->base->methods->destroy (job->base->object,
+					                              TRUE,
+					                              job->callback_data->callback_data);
 
 				g_free (job->callback_data);
 			}
@@ -593,18 +600,26 @@ brasero_io_mount_enclosing_volume (BraseroIO *self,
 				   GError **error)
 {
 	GMount *mounted;
+	GtkWindow *parent;
+	BraseroIOPrivate *priv;
 	GMountOperation *operation;
 	BraseroIOMount mount = { NULL, };
 
-	/* FIXME: need a way to get a window for the operation */
-//	operation = gtk_mount_operation_new (GTK_WINDOW (brasero_app_get_default ()));
-	operation = NULL;
+	priv = BRASERO_IO_PRIVATE (self);
+
+	if (priv->win_callback)
+		parent = priv->win_callback (priv->win_user_data);
+
+	if (parent)
+		operation = gtk_mount_operation_new (parent);
+
 	g_file_mount_enclosing_volume (file,
 				       G_MOUNT_MOUNT_NONE,
 				       operation,
 				       cancel,
 				       brasero_io_mount_enclosing_volume_cb,
 				       &mount);
+	g_object_unref (operation);
 
 	/* sleep and wait operation end */
 	while (!mount.finished && !g_cancellable_is_cancelled (cancel))
@@ -794,6 +809,8 @@ typedef struct _BraseroIOMetadataTask BraseroIOMetadataTask;
 struct _BraseroIOMetadataCached {
 	guint64 last_modified;
 	BraseroMetadataInfo *info;
+
+	guint missing_codec_used:1;
 };
 typedef struct _BraseroIOMetadataCached BraseroIOMetadataCached;
 
@@ -820,6 +837,9 @@ brasero_io_set_metadata_attributes (GFileInfo *info,
 	g_file_info_set_attribute_int32 (info, BRASERO_IO_ISRC, metadata->isrc);
 	g_file_info_set_attribute_uint64 (info, BRASERO_IO_LEN, metadata->len);
 
+	if (metadata->type)
+		g_file_info_set_content_type (info, metadata->type);
+
 	if (metadata->artist)
 		g_file_info_set_attribute_string (info, BRASERO_IO_ARTIST, metadata->artist);
 
@@ -836,9 +856,19 @@ brasero_io_set_metadata_attributes (GFileInfo *info,
 		g_file_info_set_attribute_string (info, BRASERO_IO_COMPOSER, metadata->composer);
 
 	g_file_info_set_attribute_boolean (info, BRASERO_IO_HAS_AUDIO, metadata->has_audio);
+	if (metadata->has_audio) {
+		if (metadata->channels)
+			g_file_info_set_attribute_int32 (info, BRASERO_IO_CHANNELS, metadata->channels);
+
+		if (metadata->rate)
+			g_file_info_set_attribute_int32 (info, BRASERO_IO_RATE, metadata->rate);
+
+		if (metadata->has_dts)
+			g_file_info_set_attribute_boolean (info, BRASERO_IO_HAS_DTS, TRUE);
+	}
+
 	g_file_info_set_attribute_boolean (info, BRASERO_IO_HAS_VIDEO, metadata->has_video);
 	g_file_info_set_attribute_boolean (info, BRASERO_IO_IS_SEEKABLE, metadata->is_seekable);
-
 	if (metadata->snapshot)
 		g_file_info_set_attribute_object (info, BRASERO_IO_THUMBNAIL, G_OBJECT (metadata->snapshot));
 
@@ -923,6 +953,7 @@ brasero_io_wait_for_metadata (BraseroIO *self,
 			      GCancellable *cancel,
 			      GFileInfo *info,
 			      BraseroMetadata *metadata,
+			      BraseroMetadataFlag flags,
 			      BraseroMetadataInfo *meta_info)
 {
 	gboolean result;
@@ -959,6 +990,8 @@ brasero_io_wait_for_metadata (BraseroIO *self,
 			cached->info = g_new0 (BraseroMetadataInfo, 1);
 			brasero_metadata_get_result (metadata, cached->info, NULL);
 
+			cached->missing_codec_used = (flags & BRASERO_METADATA_FLAG_MISSING) != 0;
+
 			g_queue_push_head (priv->meta_buffer, cached);
 			if (g_queue_get_length (priv->meta_buffer) > MAX_BUFFERED_META) {
 				cached = g_queue_pop_tail (priv->meta_buffer);
@@ -998,12 +1031,13 @@ brasero_io_get_metadata_info (BraseroIO *self,
 	priv = BRASERO_IO_PRIVATE (self);
 
 	mime = g_file_info_get_content_type (info);
+	BRASERO_UTILS_LOG ("Found file with type %s", mime);
+
 	if (mime
 	&& (!strncmp (mime, "image/", 6)
 	||  !strcmp (mime, "text/plain")
 	||  !strcmp (mime, "application/x-cue") /* this one make gstreamer crash */
-	||  !strcmp (mime, "application/x-cd-image")
-	||  !strcmp (mime, "application/octet-stream")))
+	||  !strcmp (mime, "application/x-cd-image")))
 		return FALSE;
 
 	BRASERO_UTILS_LOG ("Retrieving metadata info");
@@ -1021,15 +1055,24 @@ brasero_io_get_metadata_info (BraseroIO *self,
 		cached = node->data;
 		last_modified = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_STANDARD_SIZE);
 		if (last_modified == cached->last_modified) {
+			gboolean refresh_cache = FALSE;
+
+			if (flags & BRASERO_METADATA_FLAG_MISSING) {
+				/* This cached result may indicate an error and
+				 * this error could be related to the fact that
+				 * it was not first looked for with missing
+				 * codec detection. */
+				if (!cached->missing_codec_used)
+					refresh_cache = TRUE;
+			}
+
 			if (flags & BRASERO_METADATA_FLAG_THUMBNAIL) {
 				/* If there isn't any snapshot retry */
-				if (cached->info->snapshot) {
-					brasero_metadata_info_copy (meta_info, cached->info);
-					g_mutex_unlock (priv->lock_metadata);
-					return TRUE;
-				}
+				if (!cached->info->snapshot)
+					refresh_cache = TRUE;
 			}
-			else {
+
+			if (!refresh_cache) {
 				brasero_metadata_info_copy (meta_info, cached->info);
 				g_mutex_unlock (priv->lock_metadata);
 				return TRUE;
@@ -1056,6 +1099,7 @@ brasero_io_get_metadata_info (BraseroIO *self,
 					     cancel,
 					     info,
 					     metadata,
+					     flags,
 					     meta_info);
 }
 
@@ -1607,8 +1651,8 @@ brasero_io_get_file_count_process_file (BraseroIO *self,
 				if (!brasero_io_get_file_count_process_playlist (self, cancel, data, child_uri))
 					data->files_invalid ++;
 			}
-
-			data->files_invalid ++;
+			else
+				data->files_invalid ++;
 		}
 
 #endif
@@ -1714,8 +1758,8 @@ brasero_io_get_file_count_start (BraseroIO *self,
 				  (data->job.options & BRASERO_IO_INFO_FOLLOW_SYMLINK)?G_FILE_QUERY_INFO_NONE:G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,	/* follow symlinks by default*/
 				  cancel,
 				  NULL);
-
 	data->files_num ++;
+
 	if (!info) {
 		g_object_unref (file);
 		data->files_invalid ++;
@@ -1765,7 +1809,8 @@ brasero_io_get_file_count_thread (BraseroAsyncTaskManager *manager,
 		brasero_io_get_file_count_process_directory (BRASERO_IO (manager), cancel, data);
 		return BRASERO_ASYNC_TASK_RESCHEDULE;
 	}
-	else if (!data->uris) {
+
+	if (!data->uris) {
 		info = g_file_info_new ();
 
 		/* set GFileInfo information */
@@ -2147,7 +2192,7 @@ brasero_io_cancel_result (BraseroIO *self,
 	data = result->callback_data;
 	brasero_io_unref_result_callback_data (data,
 					       result->base->object,
-					       result->base->destroy,
+					       result->base->methods->destroy,
 					       TRUE);
 	brasero_io_job_result_free (result);
 }
@@ -2200,53 +2245,6 @@ brasero_io_cancel_by_base (BraseroIOJobBase *base)
 	g_object_unref (self);
 }
 
-static gboolean
-brasero_io_cancel_tasks_by_data_cb (BraseroAsyncTaskManager *manager,
-				    gpointer callback_data,
-				    gpointer user_data)
-{
-	BraseroIOJob *job = callback_data;
-
-	if (job->callback_data && job->callback_data->callback_data != user_data)
-		return FALSE;
-
-	return TRUE;
-}
-
-void
-brasero_io_cancel_by_data (gpointer callback_data)
-{
-	GSList *iter;
-	GSList *next;
-	BraseroIOPrivate *priv;
-	BraseroIO *self = brasero_io_get_default ();
-
-	priv = BRASERO_IO_PRIVATE (self);
-
-	brasero_async_task_manager_foreach_unprocessed_remove (BRASERO_ASYNC_TASK_MANAGER (self),
-							       brasero_io_cancel_tasks_by_data_cb,
-							       callback_data);
-
-	brasero_async_task_manager_foreach_active_remove (BRASERO_ASYNC_TASK_MANAGER (self),
-							  brasero_io_cancel_tasks_by_data_cb,
-							  callback_data);
-
-	/* do it afterwards in case some results slipped through */
-	for (iter = priv->results; iter; iter = next) {
-		BraseroIOJobResult *result;
-
-		result = iter->data;
-		next = iter->next;
-
-		if (result->callback_data != callback_data)
-			continue;
-
-		brasero_io_cancel_result (self, result);
-	}
-
-	g_object_unref (self);
-}
-
 struct _BraseroIOJobCompareData {
 	BraseroIOCompareCallback func;
 	const BraseroIOJobBase *base;
@@ -2290,21 +2288,76 @@ brasero_io_find_urgent (const BraseroIOJobBase *base,
 						     
 }
 
+BraseroIOJobCallbacks *
+brasero_io_register_job_methods (BraseroIOResultCallback callback,
+                                 BraseroIODestroyCallback destroy,
+                                 BraseroIOProgressCallback progress)
+{
+	BraseroIOJobCallbacks *methods;
+
+	methods = g_new0 (BraseroIOJobCallbacks, 1);
+	methods->callback = callback;
+	methods->destroy = destroy;
+	methods->progress = progress;
+
+	return methods;
+}
+
+BraseroIOJobBase *
+brasero_io_register_with_methods (GObject *object,
+                                  BraseroIOJobCallbacks *methods)
+{
+	BraseroIOJobBase *base;
+
+	base = g_new0 (BraseroIOJobBase, 1);
+	base->object = object;
+	base->methods = methods;
+	methods->ref ++;
+
+	return base;
+}
+
 BraseroIOJobBase *
 brasero_io_register (GObject *object,
 		     BraseroIOResultCallback callback,
 		     BraseroIODestroyCallback destroy,
 		     BraseroIOProgressCallback progress)
 {
-	BraseroIOJobBase *base;
+	return brasero_io_register_with_methods (object, brasero_io_register_job_methods (callback, destroy, progress));
+}
 
-	base = g_new0 (BraseroIOJobBase, 1);
-	base->object = object;
-	base->callback = callback;
-	base->destroy = destroy;
-	base->progress = progress;
+void
+brasero_io_job_base_free (BraseroIOJobBase *base)
+{
+	BraseroIOJobCallbacks *methods;
 
-	return base;
+	if (!base)
+		return;
+
+	methods = base->methods;
+	g_free (base);
+
+	methods->ref --;
+	if (methods->ref <= 0)
+		g_free (methods);
+}
+
+static int
+ brasero_io_xid_for_metadata (gpointer user_data)
+{
+	BraseroIOPrivate *priv;
+
+	priv = BRASERO_IO_PRIVATE (user_data);
+	if (priv->win_callback) {
+		int xid;
+		GtkWindow *parent;
+
+		parent = priv->win_callback (priv->win_user_data);
+		xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (gtk_widget_get_window(GTK_WIDGET (parent))));
+		return xid;
+	}
+
+	return 0;
 }
 
 static void
@@ -2323,8 +2376,10 @@ brasero_io_init (BraseroIO *object)
 	 * a thread. */
 	metadata = brasero_metadata_new ();
 	priv->metadatas = g_slist_prepend (priv->metadatas, metadata);
+	brasero_metadata_set_get_xid_callback (metadata, brasero_io_xid_for_metadata, object);
 	metadata = brasero_metadata_new ();
 	priv->metadatas = g_slist_prepend (priv->metadatas, metadata);
+	brasero_metadata_set_get_xid_callback (metadata, brasero_io_xid_for_metadata, object);
 }
 
 static gboolean
@@ -2332,9 +2387,9 @@ brasero_io_free_async_queue (BraseroAsyncTaskManager *manager,
 			     gpointer callback_data,
 			     gpointer NULL_data)
 {
-	BraseroIOJob *job = callback_data;
-
-	brasero_io_job_free (TRUE, job);
+	/* don't do anything here, the async task manager
+	 * will destroy the job anyway.
+	 */
 	return TRUE;
 }
 
@@ -2413,11 +2468,12 @@ brasero_io_finalize (GObject *object)
 			mount = iter->data;
 
 			BRASERO_UTILS_LOG ("Unmountin volume");
-			g_mount_unmount (mount,
-					 G_MOUNT_UNMOUNT_NONE,
-					 NULL,
-					 NULL,
-					 NULL);
+			g_mount_unmount_with_operation (mount,
+					 		G_MOUNT_UNMOUNT_NONE,
+							NULL,
+					 		NULL,
+					 		NULL,
+					 		NULL);
 			g_object_unref (mount);
 		}
 	}
@@ -2435,11 +2491,55 @@ brasero_io_class_init (BraseroIOClass *klass)
 	object_class->finalize = brasero_io_finalize;
 }
 
+static gboolean
+brasero_io_cancel (BraseroAsyncTaskManager *manager,
+                   gpointer callback_data,
+                   gpointer user_data)
+{
+	return TRUE;
+}
+
 void
 brasero_io_shutdown (void)
 {
+	GSList *iter, *next;
+	BraseroIOPrivate *priv;
+
+	priv = BRASERO_IO_PRIVATE (singleton);
+
+	brasero_async_task_manager_foreach_unprocessed_remove (BRASERO_ASYNC_TASK_MANAGER (singleton),
+							       brasero_io_cancel,
+							       NULL);
+
+	brasero_async_task_manager_foreach_active_remove (BRASERO_ASYNC_TASK_MANAGER (singleton),
+							  brasero_io_cancel,
+							  NULL);
+
+	/* do it afterwards in case some results slipped through */
+	for (iter = priv->results; iter; iter = next) {
+		BraseroIOJobResult *result;
+
+		result = iter->data;
+		next = iter->next;
+		brasero_io_cancel_result (singleton, result);
+	}
+
 	if (singleton) {
 		g_object_unref (singleton);
 		singleton = NULL;
 	}
 }
+
+void
+brasero_io_set_parent_window_callback (BraseroIOGetParentWinCb callback,
+                                       gpointer user_data)
+{
+	BraseroIOPrivate *priv;
+	BraseroIO *self;
+
+	self = brasero_io_get_default ();
+	priv = BRASERO_IO_PRIVATE (self);
+	priv->win_callback = callback;
+	priv->win_user_data = user_data;
+	g_object_unref (self);
+}
